# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: MIT
"""
Type stubs for unicode-segmentation-rs

This module provides Unicode text segmentation and width calculation.
"""

def graphemes(text: str, is_extended: bool) -> list[str]: ...
def grapheme_indices(text: str, is_extended: bool) -> list[tuple[int, str]]: ...
def split_word_bounds(text: str) -> list[str]: ...
def split_word_bound_indices(text: str) -> list[tuple[int, str]]: ...
def unicode_words(text: str) -> list[str]: ...
def unicode_sentences(text: str) -> list[str]: ...
def text_width(text: str) -> int: ...
def gettext_wrap(text: str, width: int) -> list[str]: ...



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: MIT

"""Unit tests for gettext wrapping"""

import unicode_segmentation_rs


class TestGettextWrap:
    """Tests for gettext PO file wrapping"""

    def test_simple_wrap(self):
        text = "This is a simple test string"
        result = unicode_segmentation_rs.gettext_wrap(text, 20)
        assert result == ["This is a simple ", "test string"]

    def test_wrap_with_cjk(self):
        text = "Hello 世界 this is a test"
        result = unicode_segmentation_rs.gettext_wrap(text, 10)
        assert result == ["Hello 世", "界 this ", "is a test"]

    def test_wrap_short_text(self):
        text = "Short"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == ["Short"]

    def test_wrap_empty_string(self):
        result = unicode_segmentation_rs.gettext_wrap("", 77)
        assert result == []

    def test_wrap_zero_width(self):
        text = "Test"
        result = unicode_segmentation_rs.gettext_wrap(text, 0)
        assert result == ["Test"]

    def test_wrap_with_punctuation(self):
        text = "Hello, world! How are you?"
        result = unicode_segmentation_rs.gettext_wrap(text, 15)
        assert result == ["Hello, world! ", "How are you?"]

    def test_wrap_with_escape_sequences(self):
        # Escape sequences should not be broken
        text = "This has \\n escape sequences \\t in it"
        result = unicode_segmentation_rs.gettext_wrap(text, 11)
        assert result == ["This has \\n", " escape ", "sequences ", "\\t in it"]

    def test_wrap_with_carriage_return_escape(self):
        text = r"foo\rbar baz"
        result = unicode_segmentation_rs.gettext_wrap(text, 5)
        assert result == [r"foo\r", "bar ", "baz"]

    def test_wrap_with_consecutive_escape_sequences(self):
        text = r"abc\n\nxyz"
        result = unicode_segmentation_rs.gettext_wrap(text, 10)
        assert result == [r"abc\n", r"\n", "xyz"]

    def test_wrap_with_doubled_backslash_before_escape(self):
        text = r"foo\\nbar"
        result = unicode_segmentation_rs.gettext_wrap(text, 5)
        assert result == [r"foo\\n", "bar"]

    def test_wrap_long_word(self):
        # Long words that don't fit should still be included
        text = "Supercalifragilisticexpialidocious"
        result = unicode_segmentation_rs.gettext_wrap(text, 20)
        assert result == [text]

    def test_wrap_default_width(self):
        # Test with typical PO file width (77 characters)
        text = "This is a longer sentence that should wrap appropriately at the standard gettext width of seventy-seven characters"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "This is a longer sentence that should wrap appropriately at the standard ",
            "gettext width of seventy-seven characters",
        ]

    def test_wrapping_spaces(self):
        """This tests that we wrap like gettext."""
        text = r"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [text]

    def test_wrapping_long_fit(self):
        text = r"bla\t12345 12345 12345 12345 12345 12 12345 12345 12345 12345 12345 12345 123"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [text]

    def test_wrapping_long_overflow(self):
        text = r"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            r"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 ",
            "1",
        ]

    def test_wrapping_long_multiline_1(self):
        text = "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1234\n1234"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1234\n",
            "1234",
        ]

    def test_wrapping_long_multiline_2(self):
        text = r"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345\n12345"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            r"bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 ",
            r"12345\n",
            "12345",
        ]

    def test_wrapping_long_escapes(self):
        text = r"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            r"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\",
            r"\\",
        ]

    def test_wrapping_cjk(self):
        text = "効率的なバグの報告はPostGISの開発を助ける本質的な方法です。最も効率的なバグ報告は、PostGIS開発者がそれを再現できるようにすることで、それの引き金となったス"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "効率的なバグの報告はPostGISの開発を助ける本質的な方法です。最も効率的なバグ報",
            "告は、PostGIS開発者がそれを再現できるようにすることで、それの引き金となったス",
        ]

    def test_wrap_combining_character_sequence(self):
        text = "Cafe\u0301 society"
        result = unicode_segmentation_rs.gettext_wrap(text, 5)
        assert result == ["Cafe\u0301 ", "society"]

    def test_wrap_standalone_combining_mark(self):
        text = "\u0301x"
        result = unicode_segmentation_rs.gettext_wrap(text, 1)
        assert result == [text]

    def test_wrap_fullwidth_punctuation_behavior(self):
        text = "你好，世界你好，世界"
        result = unicode_segmentation_rs.gettext_wrap(text, 8)
        assert result == ["你好，世", "界你好，", "世界"]

    def test_wrap_fullwidth_parentheses_behavior(self):
        text = "foo（bar）bazfoo（bar）baz"
        result = unicode_segmentation_rs.gettext_wrap(text, 12)
        assert result == ["foo（bar）", "bazfoo（bar", "）baz"]

    def test_wrap_emoji(self):
        text = 'print(ts.string_get_word_breaks("Test ❤️‍🔥 Test")) # Prints [1, 2, 3, 4, 5, '
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [text]

    def test_wrap_parenthesis_1(self):
        text = r"Konvertiert [param what] in [param type] auf die bestmögliche Weise. Der [param type] verwendet die [enum Variant.Type]-Werte.\n"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "Konvertiert [param what] in [param type] auf die bestmögliche Weise. Der ",
            r"[param type] verwendet die [enum Variant.Type]-Werte.\n",
        ]

    def test_wrap_parenthesis_2(self):
        text = r"- Eine von [Object] abgeleitete Klasse, die in [ClassDB] existiert, z. B. [Node].\n"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "- Eine von [Object] abgeleitete Klasse, die in [ClassDB] existiert, z. B. ",
            r"[Node].\n",
        ]

    def test_wrap_escape_line(self):
        text = r"%{src?%{dest?转发:入站}:出站} %{ipv6?%{ipv4?<var>IPv4</var> and <var>IPv6</var>:<var>IPv6</var>}:<var>IPv4</var>}%{proto?, 协议 %{proto#%{next?, }%{item.types?<var class=\"cbi-tooltip-container\">%{item.name}<span class=\"cbi-tooltip\">具有类型 %{item.types#%{next?, }<var>%{item}</var>} 的 ICMP</span></var>:<var>%{item.name}</var>}}}%{mark?, 标记 <var%{mark.inv? data-tooltip=\"匹配除 %{mark.num}%{mark.mask? 带有掩码 %{mark.mask}} 的 fwmarks。\":%{mark.mask? data-tooltip=\"在比较前对fwmark 应用掩码 %{mark.mask} 。\"}}>%{mark.val}</var>}%{dscp?, DSCP %{dscp.inv?<var data-tooltip=\"匹配除 %{dscp.num?:%{dscp.name}} 以外的 DSCP 类型。\">%{dscp.val}</var>:<var>%{dscp.val}</var>}}%{helper?, 助手 %{helper.inv?<var data-tooltip=\"匹配除 &quot;%{helper.name}&quot; 以外的任意助手。\">%{helper.val}</var>:<var data-tooltip=\"%{helper.name}\">%{helper.val}</var>}}"

        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "%{src?%{dest?转发:入站}:出站} %{ipv6?%{ipv4?<var>IPv4</var> and <var>IPv6</",
            "var>:<var>IPv6</var>}:<var>IPv4</var>}%{proto?, 协议 %{proto#%{next?, }%",
            r"{item.types?<var class=\"cbi-tooltip-container\">%{item.name}<span ",
            r"class=\"cbi-tooltip\">具有类型 %{item.types#%{next?, }<var>%{item}</var>} 的 ",
            "ICMP</span></var>:<var>%{item.name}</var>}}}%{mark?, 标记 <var%{mark.inv? ",
            r"data-tooltip=\"匹配除 %{mark.num}%{mark.mask? 带有掩码 %{mark.mask}} 的 ",
            r"fwmarks。\":%{mark.mask? data-tooltip=\"在比较前对fwmark 应用掩码 %",
            r"{mark.mask} 。\"}}>%{mark.val}</var>}%{dscp?, DSCP %{dscp.inv?<var data-",
            r"tooltip=\"匹配除 %{dscp.num?:%{dscp.name}} 以外的 DSCP 类型。\">%{dscp.val}</",
            "var>:<var>%{dscp.val}</var>}}%{helper?, 助手 %{helper.inv?<var data-",
            r"tooltip=\"匹配除 &quot;%{helper.name}&quot; 以外的任意助手。\">%{helper.val}",
            r"</var>:<var data-tooltip=\"%{helper.name}\">%{helper.val}</var>}}",
        ]

    def test_wrap_parenthesis_long(self):
        text = r"Must be required by a NotificationListenerService, to ensure that only the system can bind to it. See [url=https://developer.android.com/reference/android/Manifest.permission#BIND_NOTIFICATION_LISTENER_SERVICE]BIND_NOTIFICATION_LISTENER_SERVICE[/url]."
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "Must be required by a NotificationListenerService, to ensure that only the ",
            "system can bind to it. See [url=https://developer.android.com/reference/",
            "android/",
            "Manifest.permission#BIND_NOTIFICATION_LISTENER_SERVICE]BIND_NOTIFICATION_LISTENER_SERVICE[/",
            "url].",
        ]

    def test_wrap_plural_form(self):
        text = r"Plural-Forms: nplurals=3; plural=n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2;\n"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "Plural-Forms: nplurals=3; plural=n%10==1 && n%100!=11 ? 0 : n%10>=2 && ",
            r"n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2;\n",
        ]

    def test_wrap_url(self):
        text = r"Language-Team: Ukrainian <https://mirror.git.trinitydesktop.org/weblate/projects/tdepim/kmail/uk/>\n"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "Language-Team: Ukrainian <https://mirror.git.trinitydesktop.org/weblate/",
            r"projects/tdepim/kmail/uk/>\n",
        ]

    def test_wrap_escape(self):
        text = r"x: to be continued with \"do not loop\", \"loop in current folder\", and \"loop in all folders\".\nWhen trying to find unread messages:"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            r"x: to be continued with \"do not loop\", \"loop in current folder\", and ",
            r"\"loop in all folders\".\n",
            "When trying to find unread messages:",
        ]

    def test_wrap_label(self):
        text = r"You can get a copy of your Recovery Key by going to &syncBrand.shortName.label; Options on your other device, and selecting  \"My Recovery Key\" under \"Manage Account\"."
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "You can get a copy of your Recovery Key by going to ",
            "&syncBrand.shortName.label; Options on your other device, and selecting  ",
            r"\"My Recovery Key\" under \"Manage Account\".",
        ]

    def test_wrap_wide_stop(self):
        text = "在 Mastodon 上关注 [@beeware@fosstodon.org](https://fosstodon.org/@beeware)，或[加入 BeeWare 爱好者邮件列表](/zh_CN/community/keep-informed/)以获取与项目相关的更新、提示、技巧和公告。"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "在 Mastodon 上关注 [@beeware@fosstodon.org](https://fosstodon.org/@beeware)，",
            "或[加入 BeeWare 爱好者邮件列表](/zh_CN/community/keep-informed/)以获取与项目",
            "相关的更新、提示、技巧和公告。",
        ]

    def test_wrap_escape_en(self):
        text = r"By default, \":doc:`Wire Transfer </applications/finance/payment_providers/wire_transfer>`\" is the only payment provider activated, but you still have to fill out the payment details."
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            r"By default, \":doc:`Wire Transfer </applications/finance/payment_providers/",
            r"wire_transfer>`\" is the only payment provider activated, but you still have ",
            "to fill out the payment details.",
        ]

    def test_wrap_escape_localized(self):
        text = r"기본값으로 \":doc:`온라인 이체 </applications/finance/payment_providers/wire_transfer>`\"만 결제대행업체을 사용하도록 설정되어 있으나, 여기에도 결제 세부 정보를 입력해야 합니다."
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            r"기본값으로 \":doc:`온라인 이체 </applications/finance/payment_providers/",
            r"wire_transfer>`\"만 결제대행업체을 사용하도록 설정되어 있으나, 여기에도 결제 ",
            r"세부 정보를 입력해야 합니다.",
        ]

    def test_wrap_limit(self):
        text = r"Ukuba uyayiqonda into eyenzekayo, \nungaxelela i-&brandShortName; ukuba iqalise ukuthemba ufaniso lwale sayithi. \n<b>Nokuba uyayithemba isayithi, le mposiso isenokuthetha ukuba   kukho umntu \nobhucabhuca ukudibanisa kwakho.</b>"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "Ukuba uyayiqonda into eyenzekayo, \\n",
            "ungaxelela i-&brandShortName; ukuba iqalise ukuthemba ufaniso lwale sayithi. ",
            "\\n",
            "<b>Nokuba uyayithemba isayithi, le mposiso isenokuthetha ukuba   kukho umntu ",
            "\\n",
            "obhucabhuca ukudibanisa kwakho.</b>",
        ]

    def test_wrap_escape_cjk(self):
        text = "在 C# 中，请注意 ``_Process()`` 所采用的 ``delta`` 参数类型是 ``double``\\\\ 。 故当我们将其应用于旋转时，需要将其转换为 ``float`` \\\\。"
        result = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert result == [
            "在 C# 中，请注意 ``_Process()`` 所采用的 ``delta`` 参数类型是 ``double``\\\\ ",
            "。 故当我们将其应用于旋转时，需要将其转换为 ``float`` \\\\。",
        ]

    def test_wrap_round_trip_invariant(self):
        samples = [
            r"abc\n\nxyz",
            r"foo\rbar baz",
            r"foo\\nbar",
            "Cafe\u0301 society",
            "你好，世界你好，世界",
            "foo（bar）bazfoo（bar）baz",
        ]
        for text in samples:
            assert "".join(unicode_segmentation_rs.gettext_wrap(text, 8)) == text

    def test_wrap_width_invariant(self):
        samples = [
            ("This is a simple test string", 20),
            (r"abc\n\nxyz", 10),
            (r"foo\rbar baz", 5),
            ("你好，世界你好，世界", 8),
        ]
        for text, width in samples:
            for line in unicode_segmentation_rs.gettext_wrap(text, width):
                assert unicode_segmentation_rs.text_width(line) <= width



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: MIT

"""Test that all code examples in README.md work correctly"""

import unicode_segmentation_rs


class TestReadmeBasicUsage:
    """Tests for basic usage section of README"""

    def test_grapheme_clusters(self):
        """Test grapheme cluster example from README"""
        text = "Hello 👨‍👩‍👧‍👦 World"
        clusters = unicode_segmentation_rs.graphemes(text, is_extended=True)
        # Verify it works
        assert isinstance(clusters, list)
        assert len(clusters) > 0

    def test_grapheme_indices(self):
        """Test grapheme indices example from README"""
        text = "Hello 👨‍👩‍👧‍👦 World"
        indices = unicode_segmentation_rs.grapheme_indices(text, is_extended=True)
        # Verify it works
        assert isinstance(indices, list)
        assert len(indices) > 0
        assert isinstance(indices[0], tuple)

    def test_word_boundaries(self):
        """Test word boundaries example from README"""
        text = "Hello, world!"
        words = unicode_segmentation_rs.split_word_bounds(text)
        assert words == ["Hello", ",", " ", "world", "!"]

    def test_unicode_words(self):
        """Test unicode words example from README"""
        text = "Hello, world!"
        words = unicode_segmentation_rs.unicode_words(text)
        assert words == ["Hello", "world"]

    def test_word_indices(self):
        """Test word indices example from README"""
        text = "Hello, world!"
        indices = unicode_segmentation_rs.split_word_bound_indices(text)
        assert isinstance(indices, list)
        assert len(indices) > 0

    def test_sentence_segmentation(self):
        """Test sentence segmentation example from README"""
        text = "Hello world. How are you? I'm fine."
        sentences = unicode_segmentation_rs.unicode_sentences(text)
        assert isinstance(sentences, list)
        assert len(sentences) == 3

    def test_display_width(self):
        """Test display width example from README"""
        text = "Hello 世界"
        width = unicode_segmentation_rs.text_width(text)
        assert width == 10  # Hello=5, space=1, 世=2, 界=2

    def test_character_width(self):
        """Test character width examples from README"""
        assert unicode_segmentation_rs.text_width("A") == 1
        assert unicode_segmentation_rs.text_width("世") == 2
        assert unicode_segmentation_rs.text_width("\t") == 1


class TestReadmeGraphemeExamples:
    """Tests for grapheme cluster segmentation examples section"""

    def test_complex_emojis_combining_chars(self):
        """Test complex emoji and combining characters example"""
        text = "Hello 👨‍👩‍👧‍👦 नमस्ते"
        graphemes = unicode_segmentation_rs.graphemes(text, is_extended=True)
        assert isinstance(graphemes, list)
        assert len(graphemes) > 0
        assert len(graphemes) < len(text)  # Should be fewer graphemes than chars

    def test_grapheme_indices_complex(self):
        """Test grapheme indices with complex text"""
        text = "Hello 👨‍👩‍👧‍👦 नमस्ते"
        indices = unicode_segmentation_rs.grapheme_indices(text, is_extended=True)
        assert isinstance(indices, list)
        for idx, cluster in indices:
            assert isinstance(idx, int)
            assert isinstance(cluster, str)


class TestReadmeWordExamples:
    """Tests for word segmentation examples section"""

    def test_word_bounds_with_punctuation(self):
        """Test word boundaries with punctuation"""
        text = "Hello, world! How are you?"
        bounds = unicode_segmentation_rs.split_word_bounds(text)
        assert isinstance(bounds, list)
        assert len(bounds) > 0

    def test_unicode_words_extraction(self):
        """Test unicode words extraction"""
        text = "Hello, world! How are you?"
        words = unicode_segmentation_rs.unicode_words(text)
        assert isinstance(words, list)
        assert "Hello" in words
        assert "world" in words

    def test_word_boundary_indices(self):
        """Test word boundary indices"""
        text = "Hello, world! How are you?"
        indices = unicode_segmentation_rs.split_word_bound_indices(text)
        assert isinstance(indices, list)
        for idx, word in indices:
            assert isinstance(idx, int)
            assert isinstance(word, str)


class TestReadmeSentenceExamples:
    """Tests for sentence segmentation examples section"""

    def test_sentence_segmentation_multiple(self):
        """Test sentence segmentation with multiple sentences"""
        text = "Hello world. How are you? I'm fine, thanks! What about you?"
        sentences = unicode_segmentation_rs.unicode_sentences(text)
        assert isinstance(sentences, list)
        assert len(sentences) >= 3


class TestReadmeMultilingualExamples:
    """Tests for multilingual examples section"""

    def test_arabic_sentences(self):
        """Test Arabic sentence segmentation"""
        arabic = "مرحبا بك. كيف حالك؟"
        sentences = unicode_segmentation_rs.unicode_sentences(arabic)
        assert isinstance(sentences, list)
        assert len(sentences) > 0

    def test_japanese_sentences(self):
        """Test Japanese sentence segmentation"""
        japanese = "こんにちは。お元気ですか？"
        sentences = unicode_segmentation_rs.unicode_sentences(japanese)
        assert isinstance(sentences, list)
        assert len(sentences) > 0

    def test_mixed_language_words(self):
        """Test mixed language word segmentation"""
        mixed = "Hello世界! This is a test文章."
        words = unicode_segmentation_rs.unicode_words(mixed)
        assert isinstance(words, list)
        assert len(words) > 0


class TestReadmeDisplayWidthExamples:
    """Tests for display width calculation examples section"""

    def test_various_text_widths(self):
        """Test width calculation for various texts"""
        examples = [
            "Hello",
            "世界",
            "Hello 世界",
            "こんにちは",
            "🎉🎊",
            "Tab\there",
        ]

        for text in examples:
            width = unicode_segmentation_rs.text_width(text)
            assert isinstance(width, int)

    def test_character_widths_various(self):
        """Test character width for various characters"""
        chars = ["a", "A", "1", " ", "世", "界", "あ", "🎉", "\t", "\n"]
        for c in chars:
            w = unicode_segmentation_rs.text_width(c)
            assert isinstance(w, int)


class TestReadmeGettextWrapExamples:
    """Tests for gettext PO file wrapping examples section"""

    def test_basic_gettext_wrap(self):
        """Test basic gettext wrapping"""
        text = "This is a long translation string that needs to be wrapped appropriately for a gettext PO file"
        lines = unicode_segmentation_rs.gettext_wrap(text, 77)
        assert isinstance(lines, list)
        assert len(lines) > 0

    def test_gettext_wrap_cjk(self):
        """Test gettext wrapping with CJK characters"""
        text = "This translation contains 中文字符 (Chinese characters) and should wrap correctly"
        lines = unicode_segmentation_rs.gettext_wrap(text, 40)
        assert isinstance(lines, list)
        assert len(lines) > 0
        for line in lines:
            width = unicode_segmentation_rs.text_width(line)
            assert isinstance(width, int)

    def test_gettext_wrap_escape_sequences(self):
        """Test gettext wrapping preserves escape sequences"""
        text = "This has\\nline breaks\\tand tabs"
        lines = unicode_segmentation_rs.gettext_wrap(text, 20)
        assert isinstance(lines, list)
        # Verify escape sequences are preserved
        full_text = "".join(lines)
        assert "\\n" in full_text or "\\t" in full_text



# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: MIT

"""Unit tests for unicode-segmentation-rs"""

import unicode_segmentation_rs


class TestGraphemes:
    """Tests for grapheme cluster segmentation"""

    def test_simple_ascii(self):
        text = "Hello"
        result = unicode_segmentation_rs.graphemes(text, is_extended=True)
        assert result == ["H", "e", "l", "l", "o"]

    def test_emoji_zwj_sequence(self):
        # Family emoji with ZWJ (Zero Width Joiner)
        text = "👨‍👩‍👧‍👦"
        result = unicode_segmentation_rs.graphemes(text, is_extended=True)
        assert result == [text]

    def test_combining_characters(self):
        # Devanagari with combining characters
        text = "नमस्ते"
        result = unicode_segmentation_rs.graphemes(text, is_extended=True)
        # Should treat combining characters as single graphemes
        assert len(result) < len(text)

    def test_grapheme_indices(self):
        text = "Hello"
        result = unicode_segmentation_rs.grapheme_indices(text, is_extended=True)
        expected = [(0, "H"), (1, "e"), (2, "l"), (3, "l"), (4, "o")]
        assert result == expected

    def test_empty_string(self):
        result = unicode_segmentation_rs.graphemes("", is_extended=True)
        assert result == []


class TestWordSegmentation:
    """Tests for word segmentation"""

    def test_split_word_bounds_simple(self):
        text = "Hello world"
        result = unicode_segmentation_rs.split_word_bounds(text)
        assert result == ["Hello", " ", "world"]

    def test_split_word_bounds_punctuation(self):
        text = "Hello, world!"
        result = unicode_segmentation_rs.split_word_bounds(text)
        assert result == ["Hello", ",", " ", "world", "!"]

    def test_unicode_words(self):
        text = "Hello, world!"
        result = unicode_segmentation_rs.unicode_words(text)
        assert result == ["Hello", "world"]

    def test_split_word_bound_indices(self):
        text = "Hello world"
        result = unicode_segmentation_rs.split_word_bound_indices(text)
        expected = [(0, "Hello"), (5, " "), (6, "world")]
        assert result == expected

    def test_empty_string(self):
        result = unicode_segmentation_rs.unicode_words("")
        assert result == []

    def test_multilingual(self):
        text = "Hello世界"
        result = unicode_segmentation_rs.unicode_words(text)
        # Should handle mixed scripts
        assert len(result) > 0


class TestSentenceSegmentation:
    """Tests for sentence segmentation"""

    def test_simple_sentences(self):
        text = "Hello world. How are you?"
        result = unicode_segmentation_rs.unicode_sentences(text)
        assert len(result) == 2
        assert result[0] == "Hello world. "
        assert result[1] == "How are you?"

    def test_multiple_sentences(self):
        text = "First. Second! Third?"
        result = unicode_segmentation_rs.unicode_sentences(text)
        assert len(result) == 3

    def test_abbreviations(self):
        text = "Dr. Smith went home."
        result = unicode_segmentation_rs.unicode_sentences(text)
        # Should handle abbreviations correctly
        assert len(result) >= 1

    def test_empty_string(self):
        result = unicode_segmentation_rs.unicode_sentences("")
        assert result == []

    def test_arabic(self):
        text = "مرحبا بك. كيف حالك؟"
        result = unicode_segmentation_rs.unicode_sentences(text)
        assert len(result) == 2

    def test_japanese(self):
        text = "こんにちは。お元気ですか？"
        result = unicode_segmentation_rs.unicode_sentences(text)
        assert len(result) == 2


class TestDisplayWidth:
    """Tests for display width calculation"""

    def test_ascii_width(self):
        assert unicode_segmentation_rs.text_width("Hello") == 5
        assert unicode_segmentation_rs.text_width("a") == 1

    def test_cjk_width(self):
        # Chinese characters are typically 2 columns wide
        assert unicode_segmentation_rs.text_width("世界") == 4
        assert unicode_segmentation_rs.text_width("世") == 2

    def test_mixed_width(self):
        text = "Hello 世界"
        width = unicode_segmentation_rs.text_width(text)
        # "Hello" = 5, space = 1, "世界" = 4
        assert width == 10

    def test_empty_string(self):
        assert unicode_segmentation_rs.text_width("") == 0

    def test_text_width_ascii(self):
        assert unicode_segmentation_rs.text_width("a") == 1
        assert unicode_segmentation_rs.text_width("A") == 1
        assert unicode_segmentation_rs.text_width("1") == 1
        assert unicode_segmentation_rs.text_width(" ") == 1

    def test_text_width(self):
        assert unicode_segmentation_rs.text_width("世") == 2
        assert unicode_segmentation_rs.text_width("界") == 2
        assert unicode_segmentation_rs.text_width("あ") == 2

    def test_text_width_control(self):
        # Control characters should return None
        assert unicode_segmentation_rs.text_width("\t") == 1
        assert unicode_segmentation_rs.text_width("\n") == 1
        assert unicode_segmentation_rs.text_width("\r") == 1

    def test_text_width_mode(self):
        # Basic test that CJK mode works
        assert unicode_segmentation_rs.text_width("a") == 1
        assert unicode_segmentation_rs.text_width("世") == 2
        assert unicode_segmentation_rs.text_width("\t") == 1


class TestEdgeCases:
    """Tests for edge cases and special scenarios"""

    def test_only_whitespace(self):
        text = "   "
        assert unicode_segmentation_rs.graphemes(text, is_extended=True) == [
            " ",
            " ",
            " ",
        ]
        assert unicode_segmentation_rs.split_word_bounds(text) == ["   "]
        assert unicode_segmentation_rs.unicode_words(text) == []

    def test_only_punctuation(self):
        text = "!!!"
        assert unicode_segmentation_rs.unicode_words(text) == []
        assert unicode_segmentation_rs.split_word_bounds(text) == ["!", "!", "!"]

    def test_newlines(self):
        text = "Hello\nWorld"
        result = unicode_segmentation_rs.unicode_words(text)
        assert "Hello" in result
        assert "World" in result

    def test_tabs(self):
        text = "Hello\tWorld"
        result = unicode_segmentation_rs.unicode_words(text)
        assert "Hello" in result
        assert "World" in result

    def test_multiple_spaces(self):
        text = "Hello    World"
        words = unicode_segmentation_rs.unicode_words(text)
        assert words == ["Hello", "World"]


class TestPerformance:
    """Basic performance sanity checks"""

    def test_large_text_graphemes(self):
        text = "a" * 10000
        result = unicode_segmentation_rs.graphemes(text, is_extended=True)
        assert len(result) == 10000

    def test_large_text_words(self):
        text = " ".join(["word"] * 1000)
        result = unicode_segmentation_rs.unicode_words(text)
        assert len(result) == 1000

    def test_large_text_width(self):
        text = "a" * 10000
        width = unicode_segmentation_rs.text_width(text)
        assert width == 10000
